@@ -5,6 +5,8 @@
  *  Copyright (C) 2018, Bin Meng <bmeng.cn@gmail.com>
  */
 
+#define LOG_CATEGORY LOGC_EFI
+
 #include <efi_loader.h>
 #include <log.h>
 #include <mapmem.h>
@@ -25,6 +27,16 @@ efi_status_t efi_acpi_register(void)
 	ulong addr, start, end;
 	efi_status_t ret;
 
+	/*
+	 * The bloblist is already marked reserved. For now, we don't bother
+	 * marking it with EFI_ACPI_RECLAIM_MEMORY since we would need to cut a
+	 * hole in the EFI_BOOT_SERVICES_CODE region added by
+	 * add_u_boot_and_runtime(). At some point that function could create a
+	 * more detailed map.
+	 */
+	if (IS_ENABLED(CONFIG_BLOBLIST_TABLES))
+		return EFI_SUCCESS;
+
 	/* Mark space used for tables */
 	start = ALIGN_DOWN(gd->arch.table_start, EFI_PAGE_MASK);
 	end = ALIGN(gd->arch.table_end, EFI_PAGE_MASK);
@@ -41,7 +53,7 @@ efi_status_t efi_acpi_register(void)
 	}
 
 	addr = gd_acpi_start();
-	printf("EFI using ACPI tables at %lx\n", addr);
+	log_debug("EFI using ACPI tables at %lx\n", addr);
 
 	/* And expose them to our EFI payload */
 	return efi_install_configuration_table(&acpi_guid,
